@@ -1,4 +1,3 @@
-
 """
   Copyright 2011 Jeff Garzik
 
@@ -39,9 +38,11 @@
 """
 
 from requests import auth, Session, codes
+from requests.exceptions import ConnectionError
 import decimal
 import json
 import logging
+
 try:
     import urllib.parse as urlparse
 except ImportError:
@@ -58,19 +59,19 @@ class JSONRPCException(Exception):
     def __init__(self, rpc_error):
         parent_args = []
         try:
-            parent_args.append(rpc_error['message'])
+            parent_args.append(rpc_error["message"])
         except Exception:
             pass
         Exception.__init__(self, *parent_args)
         self.error = rpc_error
-        self.code = rpc_error['code'] if 'code' in rpc_error else None
-        self.message = rpc_error['message'] if 'message' in rpc_error else None
+        self.code = rpc_error["code"] if "code" in rpc_error else None
+        self.message = rpc_error["message"] if "message" in rpc_error else None
 
     def __str__(self):
-        return '%d: %s' % (self.code, self.message)
+        return "%d: %s" % (self.code, self.message)
 
     def __repr__(self):
-        return '<%s \'%s\'>' % (self.__class__.__name__, self)
+        return "<%s '%s'>" % (self.__class__.__name__, self)
 
 
 def EncodeDecimal(o):
@@ -81,12 +82,14 @@ def EncodeDecimal(o):
 
 class AuthServiceProxy(object):
     """Extension of python-jsonrpc
-    to communicate with Monero (monerod, monero-walletrpc)
+    to communicate with Monero (monerod, monero-wallet-rpc)
     """
+
     __id_count = 0
 
-    def __init__(self, service_url, service_name=None, timeout=HTTP_TIMEOUT,
-                 connection=None):
+    def __init__(
+        self, service_url, service_name=None, timeout=HTTP_TIMEOUT, connection=None
+    ):
         """
         :param service_url: http://user:passwd@host:port/json_rpc"
         :param service_name: method name of monero wallet RPC and monero daemon RPC
@@ -100,22 +103,28 @@ class AuthServiceProxy(object):
         else:
             port = self.__url.port
 
-        self.__rpc_url = (self.__url.scheme
-                          + '://' + self.__url.hostname
-                          + ':' + str(port)
-                          + self.__url.path)
+        self.__rpc_url = (
+            self.__url.scheme
+            + "://"
+            + self.__url.hostname
+            + ":"
+            + str(port)
+            + self.__url.path
+        )
 
         (user, passwd) = (self.__url.username, self.__url.password)
 
         # Digest Authentication
         authentication = None
-        log.debug('{0}, {1}'.format(user, passwd))
+        log.debug("{0}, {1}".format(user, passwd))
         if user is not None and passwd is not None:
             authentication = auth.HTTPDigestAuth(user, passwd)
 
-        headers = {'Content-Type': 'application/json',
-                   'User-Agent': USER_AGENT,
-                   'Host': self.__url.hostname}
+        headers = {
+            "Content-Type": "application/json",
+            "User-Agent": USER_AGENT,
+            "Host": self.__url.hostname,
+        }
 
         if connection:
             # Callables re-use the connection of the original proxy
@@ -126,31 +135,37 @@ class AuthServiceProxy(object):
             self.__conn.headers = headers
 
     def __getattr__(self, name):
-        if name.startswith('__') and name.endswith('__'):
+        if name.startswith("__") and name.endswith("__"):
             # Python internal stuff
             raise AttributeError
         if self.__service_name is not None:
-            name = '{0}.{1}'.format(self.__service_name, name)
-        return AuthServiceProxy(self.__service_url, name,
-                                connection=self.__conn)
+            name = "{0}.{1}".format(self.__service_name, name)
+        return AuthServiceProxy(self.__service_url, name, connection=self.__conn)
 
     def __call__(self, *args):
         AuthServiceProxy.__id_count += 1
 
-        log.debug('-{0}-> {1} {2}'.format(AuthServiceProxy.__id_count,
-                                          self.__service_name,
-                                          json.dumps(args,
-                                                     default=EncodeDecimal)))
+        log.debug(
+            "-{0}-> {1} {2}".format(
+                AuthServiceProxy.__id_count,
+                self.__service_name,
+                json.dumps(args, default=EncodeDecimal),
+            )
+        )
         # args is tuple
         # monero RPC always gets one dictionary as parameter
         if args:
             args = args[0]
 
-        postdata = json.dumps({'jsonrpc': '2.0',
-                               'method': self.__service_name,
-                               'params': args,
-                               'id': AuthServiceProxy.__id_count},
-                              default=EncodeDecimal)
+        postdata = json.dumps(
+            {
+                "jsonrpc": "2.0",
+                "method": self.__service_name,
+                "params": args,
+                "id": AuthServiceProxy.__id_count,
+            },
+            default=EncodeDecimal,
+        )
         return self._request(postdata)
 
     def batch_(self, rpc_calls):
@@ -170,36 +185,52 @@ class AuthServiceProxy(object):
         return results
 
     def _request(self, postdata):
-        log.debug('--> {}'.format(postdata))
-        r = self.__conn.post(url=self.__rpc_url,
-                             data=postdata,
-                             timeout=self.__timeout)
+        log.debug("--> {}".format(postdata))
+        try:
+            r = self.__conn.post(
+                url=self.__rpc_url, data=postdata, timeout=self.__timeout
+            )
+        except (ConnectionError) as e:
+            raise JSONRPCException(
+                {
+                    "code": -341,
+                    "message": "could not establish a connection, original error: {}".format(
+                        str(e)
+                    ),
+                }
+            )
 
         response = self._get_response(r)
-        if response.get('error', None) is not None:
-            raise JSONRPCException(response['error'])
-        elif 'result' not in response:
-            raise JSONRPCException({
-                'code': -343, 'message': 'missing JSON-RPC result'})
+        if response.get("error", None) is not None:
+            raise JSONRPCException(response["error"])
+        elif "result" not in response:
+            raise JSONRPCException({"code": -343, "message": "missing JSON-RPC result"})
         else:
-            return response['result']
+            return response["result"]
 
     def _get_response(self, r):
         if r.status_code != codes.ok:
-            raise JSONRPCException({'code': -344,
-                                   'message': 'received HTTP status code {}'.format(r.status_code)})
+            raise JSONRPCException(
+                {
+                    "code": -344,
+                    "message": "received HTTP status code {}".format(r.status_code),
+                }
+            )
         http_response = r.text
         if http_response is None:
-            raise JSONRPCException({
-                'code': -342, 'message': 'missing HTTP response from server'})
+            raise JSONRPCException(
+                {"code": -342, "message": "missing HTTP response from server"}
+            )
 
-        response = json.loads(http_response,
-                              parse_float=decimal.Decimal)
-        if 'error' in response and response.get('error', None) is None:
-            log.error('error: {}'.format(response))
-            log.debug('<-{0}- {1}'.format(response['id'],
-                                          json.dumps(response['result'],
-                                                     default=EncodeDecimal)))
+        response = json.loads(http_response, parse_float=decimal.Decimal)
+        if "error" in response and response.get("error", None) is None:
+            log.error("error: {}".format(response))
+            log.debug(
+                "<-{0}- {1}".format(
+                    response["id"],
+                    json.dumps(response["result"], default=EncodeDecimal),
+                )
+            )
         else:
-            log.debug('<-- {}'.format(response))
+            log.debug("<-- {}".format(response))
         return response
